<?
/****************************************************************************
S7_FUNCTION
 + ist eine Funktionssammlung fr den Zugriff auf S7-Instanzen

Release: 0.6
ReleaseDate: 21.07.2012

ReleaseNote:
30.04.2009 - R0.3 - Gusenbauer
 + Function S7_VAR_IDs_READ geandert

02.06.2012 - R0.4 - Gusenbauer
 + Function S7_WRITE eingefuehrt

10.06.2012 - R0.5 - Gusenbauer
 + Check der Instanzen ob OKAY, ansonsten Abbruch
	- S7_VAR_IDs_READ
	- S7_WRITE

10.06.2012 - R0.6 - Gusenbauer
 + Auslesen der Verbindungsinstanz hat sich mit der IPS2.6 geaendert !!
   ==> daher wird ab 2.6 die "ConnectionID" verwendet
   ==> bis 2.5 aber die "ParentID"

*****************************************************************************/

include_once ('./common_function.php');


/****************************************************************************
S7_VAR_IDs_READ,
  liest das uebergebene Variablenarray "S7_VAR_IDs" von der "S7_Instanz_ID"
  und legt die Variablen entsprechend dem "S7_Path" ab

Example:    //index0   = VARIABLENBEZEICHNUNG in IPS
            //index1.0 = TYP          (0=bit, 1=byte, 2=word, 3=dword,
				//                         4=ShortInt, 5=SmallInt, 6=Integer, 7=Real)
            //index1.1 = AREA         (0=SystemInfo, 1=SystemFlags,
				//                         2=AnalogInput CPU200, 3=AnalogOutput CPU200,
				//                         4=Input, 5=Output, 6=Flags, 7=Datablock,
				//                         8=InstanzDaten, 9=LokaleDaten, 10=UnknownedArea
				//                         11=Zaehler, 12=Timer, 13=PEW/PAW)
            //index1.2 = AREAADDRESS  (Baustein)
            //index1.3 = ADDRES       (StartByte)
            //index1.4 = BIT          (Bit = wird nur bei TYP=0 verarbeitet)
            //index1.5 = CONVERT-S7-Value vor schreiben auf die IPS-Variable
            //                        (0=unverndert, 1=Dec2Hex, 2=Hex2Dec)

            $S7_VAR_IDs = array("M0.1"   => array(0,6,0,0,1,0),
				                    "E41.1"  => array(0,4,0,41,1,0));
            S7_VAR_IDs_READ(123456,"S7/TEST",$S7_VAR_IDs);

ReleaseNotes:
02.04.2009 Gusenbauer
 + Function erstellt und geprft
07.04.2009 Gusenbauer
 + Wenn Lesefehler oder Zuweisungsfehler wird ber ERR_COUNT gezaehlt
	und zurueckgegeben!!
18.04.2009 Gusenbauer
 + Pruefung eingefuehrt welche bei M0.1 die Leseroutine abbricht
   wenn 0 gelesen wird (am sinnvollsten als erste Variable uebergeben)
30.04.2009 Gusenbauer
 + Baustein um REAL erweitert und wird als FLOAT abgelegt, Anfrage von MICHELH
10.06.2012 Gusenbauer
 + Check der Instanzen ob OKAY,
	sonst Abbruch und RET >= 1000
	RET: 9999 = Instanzefehler
		  1000 = S7-Instanzenfehler
		  1001 = S7-Verbindungsinstanzenfehler
		  
*****************************************************************************/
function S7_VAR_IDs_READ($S7_Instanz_ID,$S7_Path,$S7_VAR_IDs)
{
  CheckCreateCategory($S7_Path);
  $err_count = 0;

  while($S7_VAR_ID = each($S7_VAR_IDs))
    {
		   //Wenn Variable noch nicht existiert
			if ($S7_VAR_ID[1][0] == 0)
				{
             $VarID = CheckCreateVariable($S7_Path . "/" . $S7_VAR_ID[0],0);  //boolean
				}
			else if ($S7_VAR_ID[1][0] == 7)
				{
             $VarID = CheckCreateVariable($S7_Path . "/" . $S7_VAR_ID[0],2);  //float
				}
			else
				{
             $VarID = CheckCreateVariable($S7_Path . "/" . $S7_VAR_ID[0],1);  //integer
				}

         S7_SetType        ($S7_Instanz_ID,$S7_VAR_ID[1][0]);
         S7_SetArea        ($S7_Instanz_ID,$S7_VAR_ID[1][1]);
         S7_SetAreaAddress ($S7_Instanz_ID,$S7_VAR_ID[1][2]);
         S7_SetAddress     ($S7_Instanz_ID,$S7_VAR_ID[1][3]);
         S7_SetBit         ($S7_Instanz_ID,$S7_VAR_ID[1][4]);

			//Polltimer sicherheitshalber ausschalten
         S7_SetPoller      ($S7_Instanz_ID,0);

			//Instanzkonfiguration speichern
	      if (IPS_ApplyChanges($S7_Instanz_ID) == false)
	         {
             IPS_LogMessage("S7_FUNCTION",
		                 "ERROR: S7-Instanzenaenderung konnte nicht gespeichert werden !! --> ABBRUCH !!");

			    echo ("ERROR: S7-Instanzenaenderung konnte nicht gespeichert werden !! ==> S7-Variable: " . $S7_VAR_ID[0] . "\r\n");
             $err_count = 9999;
			    return $err_count;
				}

	      //Check ob S7-Instanz okay ???
	      $instance = IPS_GetInstance($S7_Instanz_ID);
         if ($instance['InstanceStatus'] >= 104)
            {
             IPS_LogMessage("S7_FUNCTION",
		                 "ERROR: S7-Instanzenfehler aufgetreten !! --> ABBRUCH !!");

			    echo ("ERROR: S7-Instanzenfehler aufgetreten !! ==> S7-Variable: " . $S7_VAR_ID[0] . "\r\n");
             $err_count = 1000;
			    return $err_count;
            }

	      //Check ob Verbindungsinstanz zur Steuerung wirklich steht ???
	      $S7_ParentInstanz_ID = 0;
         if ((float) IPS_GetKernelVersion() >= 2.6)
			   {
             $S7_ParentInstanz_ID = $instance['ConnectionID'];
            }
		   else
		 	   {
             $S7_ParentInstanz_ID = $instance['ParentID'];
			  }

		   if ($S7_ParentInstanz_ID == 0)
		  	   {
			    IPS_LogMessage ("S7_FUNCTION", "Error: Kann ID der Verbindungsinstanz nicht ermitteln !!");
			    return false;
			   }

	      $instance = IPS_GetInstance($S7_ParentInstanz_ID);
         if ($instance['InstanceStatus'] >= 104)
            {
             IPS_LogMessage("S7_FUNCTION",
		                 "ERROR: S7-Verbindungsinstanzenfehler aufgetreten !! --> ABBRUCH !!");

 			    echo ("ERROR: S7-Verbindungsinstanzenfehler aufgetreten !! ==> S7-Variable: " . $S7_VAR_ID[0] . "\r\n");
             $err_count = 1001;
			    return $err_count;
            }

			if (S7_RequestRead($S7_Instanz_ID) == true)
				{
			    $VarStatusID = IPS_GetStatusVariableID($S7_Instanz_ID,"Value");

				 //Check ob eine konvertierung stattfinden soll
				 if ($S7_VAR_ID[1][5] == 1)
					 {
                 $S7_READ_VALUE = intval(hexdec(GetValue($VarStatusID)));
					 }
				 else if ($S7_VAR_ID[1][5] == 2)
					 {
                 $S7_READ_VALUE = intval(dechex(GetValue($VarStatusID)));
					 }
				 else
					 {
                 $S7_READ_VALUE = GetValue($VarStatusID);
					 }

				//Pruefung ob Variable M0.1 entspricht und wenn hier der Wert 0, dann lesen abbrechen
				//nicht so schoen aber Wirkungsvoll, denn beim Anlauf immer wieder 0 fuer alle Variablen gelesen

				if ($S7_VAR_ID[0] == "M0.1")
					{
					if($S7_READ_VALUE == false)
					  {
                  $err_count++;
  					   echo "FEHLER: M0.1 ist mit 0 gelesen worden --> Abbruch der Leseroutine\r\n";
						return $err_count;
					  }
					}

				//----------------------------------------

			   //S7_VAR_ID - Wert aktualisieren
			   if ($S7_VAR_ID[1][0] == 0)
				   {
				    if (SetValueBoolean($VarID,$S7_READ_VALUE) == false)
						 {
					     echo "FEHLER: Wert '" . $S7_READ_VALUE . "' konnte mit SetValueBoolean auf die ID: " . $VarID . " zuwegwiesen werden (" . $S7_Path . "/" . $S7_VAR_ID[0] . ")\r\n";
						 }
  				   }
			   else if ($S7_VAR_ID[1][0] == 7)  //REAL
			      {
				    if (SetValueFloat($VarID,$S7_READ_VALUE) == false)
						 {
					     echo "FEHLER: Wert '" . $S7_READ_VALUE . "' konnte mit SetValueFloat auf die ID: " . $VarID . " zuwegwiesen werden (" . $S7_Path . "/" . $S7_VAR_ID[0] . ")\r\n";
                    $err_count++;
						 }
				   }
			   else
			      {
				    if (SetValueInteger($VarID,$S7_READ_VALUE) == false)
						 {
					     echo "FEHLER: Wert '" . $S7_READ_VALUE . "' konnte mit SetValueInteger auf die ID: " . $VarID . " zuwegwiesen werden (" . $S7_Path . "/" . $S7_VAR_ID[0] . ")\r\n";
                    $err_count++;
						 }
				   }
			  }
			else
			  {
			  echo ("WARNING: Fehler beim lesen der S7-Variable: " . $S7_VAR_ID[0] . "\r\n");
			  $err_count++;
			  }
    }

   if ($err_count>= 1)
		{
		return $err_count;
		}
	else
		{
		return false;
		}

}




/****************************************************************************
S7_WRITE,
  schreibt einen Wert an die S7 ueber die angegeben S7-Instanz

Paramter:   INSTANZ      Gibt die Instanz an ueber die geschrieben wird
            TYP          (0=bit, 1=byte, 2=word, 3=dword,
				              4=ShortInt, 5=SmallInt, 6=Integer, 7=Real)
            AREA         (0=SystemInfo, 1=SystemFlags,
				              2=AnalogInput CPU200, 3=AnalogOutput CPU200,
				              4=Input, 5=Output, 6=Flags, 7=Datablock,
				              8=InstanzDaten, 9=LokaleDaten, 10=UnknownedArea
				              11=Zaehler, 12=Timer, 13=PEW/PAW)
            AREAADDRESS  (Baustein)
            ADDRES       (StartByte)
            BIT          (Bit = wird nur bei TYP=0 verarbeitet)

				VALUE        Wert der geschrieben werden soll

Example:    S7_WRITE(123456,0,6,0,0,1,true);  //M0.1=1 setzen


ReleaseNotes:
02.06.2012 Gusenbauer
 + Function erstellt und geprft
10.06.2012 Gusenbauer
 + Check der Instanzen ob OKAY,
	sonst Abbruch und RET = FALSE

*****************************************************************************/

function S7_WRITE($S7_Instanz_ID, $S7_Typ, $S7_Area, $S7_AreaAddress, $S7_Address, $S7_Bit, $S7_WriteValue)
{
	$ret = false;

	//ParentInstanz umkonfigurieren
   S7_SetType        ($S7_Instanz_ID,$S7_Typ);
   S7_SetArea        ($S7_Instanz_ID,$S7_Area);
   S7_SetAreaAddress ($S7_Instanz_ID,$S7_AreaAddress);
   S7_SetAddress     ($S7_Instanz_ID,$S7_Address);
   S7_SetBit         ($S7_Instanz_ID,$S7_Bit);

	//Polltimer sicherheitshalber ausschalten
   S7_SetPoller      ($S7_Instanz_ID,0);

   //Instanzkonfiguration speichern und Schreibauftrag ausloesen
	if (IPS_ApplyChanges($S7_Instanz_ID) == true)
	   {

	    //Check ob S7-Instanz okay ???
	    $instance = IPS_GetInstance($S7_Instanz_ID);
       if ($instance['InstanceStatus'] >= 104)
          {
           IPS_LogMessage("S7_FUNCTION",
		                 "ERROR: S7-Instanzenfehler aufgetreten !! --> ABBRUCH !!");
		     return false;
          }

	    //Check ob Verbindungsinstanz zur Steuerung wirklich steht ???
	    $S7_ParentInstanz_ID = 0;
       if ((float) IPS_GetKernelVersion() >= 2.6)
			 {
           $S7_ParentInstanz_ID = $instance['ConnectionID'];
          }
		 else
		 	 {
           $S7_ParentInstanz_ID = $instance['ParentID'];
			}

		 if ($S7_ParentInstanz_ID == 0)
		  	 {
			  IPS_LogMessage ("S7_FUNCTION", "Error: Kann ID der Verbindungsinstanz nicht ermitteln !!");
			  return false;
			 }

	    $instance = IPS_GetInstance($S7_ParentInstanz_ID);
       if ($instance['InstanceStatus'] >= 104)
          {
           IPS_LogMessage("S7_FUNCTION",
		                 "ERROR: S7-Verbindungsinstanzenfehler aufgetreten !! --> ABBRUCH !!");
		     return false;
          }


		 if ($S7_Typ == 0)
			 {
	        $ret = S7_WriteBit($S7_Instanz_ID, ConvertToBoolean($S7_WriteValue));
			 }
		 else if ($S7_Typ == 1)
			 {
	        $ret = S7_WriteByte($S7_Instanz_ID, $S7_WriteValue);
			 }
		 else if ($S7_Typ == 2)
			 {
	        $ret = S7_WriteWord($S7_Instanz_ID, $S7_WriteValue);
			 }
		 else if ($S7_Typ == 3)
			 {
	        $ret = S7_WriteDWord($S7_Instanz_ID, $S7_WriteValue);
			 }
		 else if ($S7_Typ == 4)
			 {
	        $ret = S7_WriteShortInt($S7_Instanz_ID, $S7_WriteValue);
			 }
		 else if ($S7_Typ == 5)
			 {
	        $ret = S7_WriteSmallInt($S7_Instanz_ID, $S7_WriteValue);
			 }
		 else if ($S7_Typ == 6)
			 {
	        $ret = S7_WriteInteger($S7_Instanz_ID, $S7_WriteValue);
			 }
		 else if ($S7_Typ == 7)
			 {
	        $ret = S7_WriteReal($S7_Instanz_ID, $S7_WriteValue);
			 }
		 else
			 {
			  $ret = false;
			 }


		 if ($ret == false)
			 {
		     IPS_LogMessage("S7_WRITE", "Fehler Schreiben der Instanz: "
			                             . $S7_Instanz_ID . " ==> ("
			                             . $S7_Typ . ","
												  . $S7_Area . ","
												  . $S7_AreaAddress . ","
												  . $S7_Address . ","
												  . $S7_Bit . ") = "
												  . $S7_WriteValue
                                      );
			 }

      }
	else
		{
		 IPS_LogMessage("S7_WRITE", "Fehler beim umkonfigurieren der Instanz : " . $S7_Instanz_ID);
		 $ret = false;
		}


	return $ret;
}

//****************************************************************************

?>